@@ -37,25 +37,30 @@ import click
 from flask import Flask, Blueprint, make_response, request, send_from_directory
 
 from pygeoapi.api import API
-from pygeoapi.util import get_mimetype, yaml_load
+from pygeoapi.util import get_mimetype, yaml_load, get_api_rules
 
 
-CONFIG = None
-
 if 'PYGEOAPI_CONFIG' not in os.environ:
     raise RuntimeError('PYGEOAPI_CONFIG environment variable not set')
 
 with open(os.environ.get('PYGEOAPI_CONFIG'), encoding='utf8') as fh:
     CONFIG = yaml_load(fh)
 
+API_RULES = get_api_rules(CONFIG)
+
 STATIC_FOLDER = 'static'
 if 'templates' in CONFIG['server']:
     STATIC_FOLDER = CONFIG['server']['templates'].get('static', 'static')
 
 APP = Flask(__name__, static_folder=STATIC_FOLDER, static_url_path='/static')
-APP.url_map.strict_slashes = False
+APP.url_map.strict_slashes = API_RULES.strict_slashes
 
-BLUEPRINT = Blueprint('pygeoapi', __name__, static_folder=STATIC_FOLDER)
+BLUEPRINT = Blueprint(
+    'pygeoapi',
+    __name__,
+    static_folder=STATIC_FOLDER,
+    url_prefix=API_RULES.get_url_prefix('flask')
+)
 
 # CORS: optionally enable from config.
 if CONFIG['server'].get('cors', False):
@@ -179,9 +184,11 @@ def collection_queryables(collection_id=None):
 
 
 @BLUEPRINT.route('/collections/<path:collection_id>/items',
-                 methods=['GET', 'POST'])
+                 methods=['GET', 'POST', 'OPTIONS'],
+                 provide_automatic_options=False)
 @BLUEPRINT.route('/collections/<path:collection_id>/items/<path:item_id>',
-                 methods=['GET', 'PUT', 'DELETE'])
+                 methods=['GET', 'PUT', 'DELETE', 'OPTIONS'],
+                 provide_automatic_options=False)
 def collection_items(collection_id, item_id=None):
     """
     OGC API collections items endpoint
@@ -205,6 +212,9 @@ def collection_items(collection_id, item_id=None):
                 else:
                     return get_response(
                         api_.post_collection_items(request, collection_id))
+        elif request.method == 'OPTIONS':
+            return get_response(
+                api_.manage_collection_item(request, 'options', collection_id))
 
     elif request.method == 'DELETE':
         return get_response(
@@ -214,6 +224,10 @@ def collection_items(collection_id, item_id=None):
         return get_response(
             api_.manage_collection_item(request, 'update',
                                         collection_id, item_id))
+    elif request.method == 'OPTIONS':
+        return get_response(
+            api_.manage_collection_item(request, 'options',
+                                        collection_id, item_id))
     else:
         return get_response(
             api_.get_collection_item(request, collection_id, item_id))
@@ -405,11 +419,13 @@ def get_job_result_resource(job_id, resource):
 @BLUEPRINT.route('/collections/<path:collection_id>/position')
 @BLUEPRINT.route('/collections/<path:collection_id>/area')
 @BLUEPRINT.route('/collections/<path:collection_id>/cube')
+@BLUEPRINT.route('/collections/<path:collection_id>/radius')
 @BLUEPRINT.route('/collections/<path:collection_id>/trajectory')
 @BLUEPRINT.route('/collections/<path:collection_id>/corridor')
 @BLUEPRINT.route('/collections/<path:collection_id>/instances/<instance_id>/position')  # noqa
 @BLUEPRINT.route('/collections/<path:collection_id>/instances/<instance_id>/area')  # noqa
 @BLUEPRINT.route('/collections/<path:collection_id>/instances/<instance_id>/cube')  # noqa
+@BLUEPRINT.route('/collections/<path:collection_id>/instances/<instance_id>/radius')  # noqa
 @BLUEPRINT.route('/collections/<path:collection_id>/instances/<instance_id>/trajectory')  # noqa
 @BLUEPRINT.route('/collections/<path:collection_id>/instances/<instance_id>/corridor')  # noqa
 def get_collection_edr_query(collection_id, instance_id=None):
