@@ -2,7 +2,7 @@
 #
 # Authors: Tom Kralidis <tomkralidis@gmail.com>
 #
-# Copyright (c) 2022 Tom Kralidis
+# Copyright (c) 2023 Tom Kralidis
 # Copyright (c) 2021 Francesco Bartoli
 #
 # Permission is hereby granted, free of charge, to any person
@@ -32,17 +32,16 @@ from typing import Dict
 from collections import OrderedDict
 import json
 import logging
-from urllib.parse import urlparse
+import uuid
 
 from elasticsearch import Elasticsearch, exceptions, helpers
-from elasticsearch.client.indices import IndicesClient
 from elasticsearch_dsl import Search, Q
 
 from pygeoapi.provider.base import (BaseProvider, ProviderConnectionError,
                                     ProviderQueryError,
                                     ProviderItemNotFoundError)
 from pygeoapi.models.cql import CQLModel, get_next_node
-from pygeoapi.util import get_envelope
+from pygeoapi.util import get_envelope, crs_transform
 
 
 LOGGER = logging.getLogger(__name__)
@@ -62,50 +61,26 @@ class ElasticsearchProvider(BaseProvider):
 
         super().__init__(provider_def)
 
+        self.select_properties = []
+
         self.es_host, self.index_name = self.data.rsplit('/', 1)
 
         LOGGER.debug('Setting Elasticsearch properties')
-        self.is_gdal = False
 
         LOGGER.debug(f'host: {self.es_host}')
         LOGGER.debug(f'index: {self.index_name}')
 
-        self.type_name = 'FeatureCollection'
-        self.url_parsed = urlparse(self.es_host)
-
-        self.select_properties = []
-
         LOGGER.debug('Connecting to Elasticsearch')
-
-        if self.url_parsed.port is None:  # proxy to default HTTP(S) port
-            if self.url_parsed.scheme == 'https':
-                port = 443
-            else:
-                port = 80
-        else:  # was set explictly
-            port = self.url_parsed.port
-
-        url_settings = {
-            'scheme': self.url_parsed.scheme,
-            'host': self.url_parsed.hostname,
-            'port': port
-        }
-
-        if self.url_parsed.path:
-            url_settings['url_prefix'] = self.url_parsed.path
-
-        LOGGER.debug(f'URL settings: {url_settings}')
-        LOGGER.debug('Connecting to Elasticsearch')
-        self.es = Elasticsearch([url_settings])
+        self.es = Elasticsearch(self.es_host)
         if not self.es.ping():
-            msg = 'Cannot connect to Elasticsearch'
+            msg = f'Cannot connect to Elasticsearch: {self.es_host}'
             LOGGER.error(msg)
             raise ProviderConnectionError(msg)
 
         LOGGER.debug('Determining ES version')
         v = self.es.info()['version']['number'][:3]
-        if float(v) < 7:
-            msg = 'only ES 7+ supported'
+        if float(v) < 8:
+            msg = 'only ES 8+ supported'
             LOGGER.error(msg)
             raise ProviderConnectionError(msg)
 
@@ -124,9 +99,9 @@ class ElasticsearchProvider(BaseProvider):
         """
 
         fields_ = {}
-        ic = IndicesClient(self.es)
-        ii = ic.get(index=self.index_name)
+        ii = self.es.indices.get(index=self.index_name)
 
+        LOGGER.debug(f'Response: {ii}')
         try:
             if '*' not in self.index_name:
                 p = ii[self.index_name]['mappings']['properties']['properties']
@@ -135,9 +110,9 @@ class ElasticsearchProvider(BaseProvider):
                 index_name_ = list(ii.keys())[0]
                 p = ii[index_name_]['mappings']['properties']['properties']
         except KeyError:
-            LOGGER.debug('ES index looks generated by GDAL')
-            self.is_gdal = True
-            p = ii[self.index_name]['mappings']
+            LOGGER.warning('Trying for alias')
+            alias_name = next(iter(ii))
+            p = ii[alias_name]['mappings']['properties']['properties']
         except IndexError:
             LOGGER.warning('could not get fields; returning empty set')
             return {}
@@ -155,6 +130,7 @@ class ElasticsearchProvider(BaseProvider):
 
         return fields_
 
+    @crs_transform
     def query(self, offset=0, limit=10, resulttype='results',
               bbox=[], datetime_=None, properties=[], sortby=[],
               select_properties=[], skip_geometry=False, q=None,
@@ -305,7 +281,7 @@ class ElasticsearchProvider(BaseProvider):
                 'includes': list(map(self.mask_prop, all_properties))
             }
 
-            query['_source']['includes'].append(self.mask_prop(self.id_field))
+            query['_source']['includes'].append('id')
             query['_source']['includes'].append('type')
             query['_source']['includes'].append('geometry')
 
@@ -336,13 +312,15 @@ class ElasticsearchProvider(BaseProvider):
                             next(gen)
                     except StopIteration:
                         break
-                results['hits']['total'] = \
-                    len(results['hits']['hits']) + offset
+
+                matched = len(results['hits']['hits']) + offset
+                returned = len(results['hits']['hits'])
             else:
-                results = self.es.search(index=self.index_name,
-                                         from_=offset, size=limit,
-                                         body=query)
-                results['hits']['total'] = results['hits']['total']['value']
+                es_results = self.es.search(index=self.index_name,
+                                            from_=offset, size=limit, **query)
+                results = es_results
+                matched = es_results['hits']['total']['value']
+                returned = len(es_results['hits']['hits'])
 
         except exceptions.ConnectionError as err:
             LOGGER.error(err)
@@ -354,12 +332,12 @@ class ElasticsearchProvider(BaseProvider):
             LOGGER.error(err)
             raise ProviderQueryError()
 
-        feature_collection['numberMatched'] = results['hits']['total']
+        feature_collection['numberMatched'] = matched
 
         if resulttype == 'hits':
             return feature_collection
 
-        feature_collection['numberReturned'] = len(results['hits']['hits'])
+        feature_collection['numberReturned'] = returned
 
         LOGGER.debug('serializing features')
         for feature in results['hits']['hits']:
@@ -368,6 +346,7 @@ class ElasticsearchProvider(BaseProvider):
 
         return feature_collection
 
+    @crs_transform
     def get(self, identifier, **kwargs):
         """
         Get ES document by id
@@ -382,7 +361,7 @@ class ElasticsearchProvider(BaseProvider):
             result = self.es.get(index=self.index_name, id=identifier)
             LOGGER.debug('Serializing feature')
             feature_ = self.esdoc2geojson(result)
-        except exceptions.NotFoundError as err:
+        except Exception as err:
             LOGGER.debug(f'Not found via ES id query: {err}')
             LOGGER.debug('Trying via a real query')
 
@@ -391,15 +370,16 @@ class ElasticsearchProvider(BaseProvider):
                     'bool': {
                         'filter': [{
                             'match_phrase': {
-                                self.mask_prop(self.id_field): identifier
+                                '_id': identifier
                             }
                         }]
                     }
                 }
             }
 
+            LOGGER.debug(f'Query: {query}')
             try:
-                result = self.es.search(index=self.index_name, body=query)
+                result = self.es.search(index=self.index_name, **query)
                 if len(result['hits']['hits']) == 0:
                     LOGGER.error(err)
                     raise ProviderItemNotFoundError(err)
@@ -423,7 +403,12 @@ class ElasticsearchProvider(BaseProvider):
         :returns: identifier of created item
         """
 
-        identifier, json_data = self._load_and_prepare_item(item)
+        identifier, json_data = self._load_and_prepare_item(
+            item, accept_missing_identifier=True)
+        if identifier is None:
+            # If there is no incoming identifier, allocate a random one
+            identifier = str(uuid.uuid4())
+            json_data["id"] = identifier
 
         LOGGER.debug(f'Inserting data with identifier {identifier}')
         _ = self.es.index(index=self.index_name, id=identifier, body=json_data)
@@ -475,24 +460,16 @@ class ElasticsearchProvider(BaseProvider):
         feature_ = {}
         feature_thinned = {}
 
-        if 'properties' not in doc['_source']:
-            LOGGER.debug('Looks like a GDAL ES 7 document')
-            id_ = doc['_source'][self.id_field]
-            if 'type' not in doc['_source']:
-                feature_['id'] = id_
-                feature_['type'] = 'Feature'
-            feature_['geometry'] = doc['_source'].get('geometry')
-            feature_['properties'] = {}
-            for key, value in doc['_source'].items():
-                if key == 'geometry':
-                    continue
-                feature_['properties'][key] = value
-        else:
-            LOGGER.debug('Looks like true GeoJSON document')
-            feature_ = doc['_source']
+        LOGGER.debug('Fetching id and geometry from GeoJSON document')
+        feature_ = doc['_source']
+
+        if self.id_field in doc['_source']['properties']:
             id_ = doc['_source']['properties'][self.id_field]
-            feature_['id'] = id_
-            feature_['geometry'] = doc['_source'].get('geometry')
+        else:
+            id_ = doc['_source'].get('id', doc['_id'])
+
+        feature_['id'] = id_
+        feature_['geometry'] = doc['_source'].get('geometry')
 
         if self.properties or self.select_properties:
             LOGGER.debug('Filtering properties')
@@ -525,10 +502,7 @@ class ElasticsearchProvider(BaseProvider):
         :returns: masked property name
         """
 
-        if self.is_gdal:
-            return property_name
-        else:
-            return f'properties.{property_name}'
+        return f'properties.{property_name}'
 
     def get_properties(self):
         all_properties = []
