@@ -3,7 +3,7 @@
 # Authors: Matthew Perry <perrygeo@gmail.com>
 #
 # Copyright (c) 2018 Matthew Perry
-# Copyright (c) 2021 Tom Kralidis
+# Copyright (c) 2022 Tom Kralidis
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -34,6 +34,7 @@ import os
 import uuid
 
 from pygeoapi.provider.base import BaseProvider, ProviderItemNotFoundError
+from pygeoapi.util import crs_transform
 
 LOGGER = logging.getLogger(__name__)
 
@@ -76,14 +77,23 @@ class GeoJSONProvider(BaseProvider):
         :returns: dict of fields
         """
 
+        fields = {}
         LOGGER.debug('Treating all columns as string types')
         if os.path.exists(self.data):
             with open(self.data) as src:
                 data = json.loads(src.read())
-            fields = {}
-            for f in data['features'][0]['properties'].keys():
-                fields[f] = {'type': 'string'}
-            return fields
+            for key, value in data['features'][0]['properties'].items():
+                if isinstance(value, float):
+                    type_ = 'number'
+                elif isinstance(value, int):
+                    type_ = 'integer'
+                else:
+                    type_ = 'string'
+
+                fields[key] = {'type': type_}
+        else:
+            LOGGER.warning(f'File {self.data} does not exist.')
+        return fields
 
     def _load(self, skip_geometry=None, properties=[], select_properties=[]):
         """Load and validate the source GeoJSON file
@@ -97,6 +107,7 @@ class GeoJSONProvider(BaseProvider):
             with open(self.data) as src:
                 data = json.loads(src.read())
         else:
+            LOGGER.warning(f'File {self.data} does not exist.')
             data = {
                 'type': 'FeatureCollection',
                 'features': []}
@@ -120,13 +131,14 @@ class GeoJSONProvider(BaseProvider):
                                    if k in set(self.properties) | set(select_properties)}  # noqa
         return data
 
-    def query(self, startindex=0, limit=10, resulttype='results',
+    @crs_transform
+    def query(self, offset=0, limit=10, resulttype='results',
               bbox=[], datetime_=None, properties=[], sortby=[],
               select_properties=[], skip_geometry=False, q=None, **kwargs):
         """
         query the provider
 
-        :param startindex: starting record to return (default 0)
+        :param offset: starting record to return (default 0)
         :param limit: number of records to return (default 10)
         :param resulttype: return results or hit limit (default results)
         :param bbox: bounding box [minx,miny,maxx,maxy]
@@ -149,11 +161,12 @@ class GeoJSONProvider(BaseProvider):
         if resulttype == 'hits':
             data['features'] = []
         else:
-            data['features'] = data['features'][startindex:startindex+limit]
+            data['features'] = data['features'][offset:offset+limit]
             data['numberReturned'] = len(data['features'])
 
         return data
 
+    @crs_transform
     def get(self, identifier, **kwargs):
         """
         query the provider by id
@@ -168,7 +181,7 @@ class GeoJSONProvider(BaseProvider):
             if str(feature.get('id')) == identifier:
                 return feature
         # default, no match
-        err = 'item {} not found'.format(identifier)
+        err = f'item {identifier} not found'
         LOGGER.error(err)
         raise ProviderItemNotFoundError(err)
 
@@ -227,4 +240,4 @@ class GeoJSONProvider(BaseProvider):
             dst.write(json.dumps(all_data))
 
     def __repr__(self):
-        return '<GeoJSONProvider> {}'.format(self.data)
+        return f'<GeoJSONProvider> {self.data}'
