@@ -37,6 +37,7 @@ from pymongo import GEOSPHERE
 from pymongo import ASCENDING, DESCENDING
 from pymongo.collection import ObjectId
 from pygeoapi.provider.base import BaseProvider, ProviderItemNotFoundError
+from pygeoapi.util import crs_transform
 
 LOGGER = logging.getLogger(__name__)
 
@@ -109,6 +110,7 @@ class MongoProvider(BaseProvider):
 
         return featurelist, matchCount
 
+    @crs_transform
     def query(self, offset=0, limit=10, resulttype='results',
               bbox=[], datetime_=None, properties=[], sortby=[],
               select_properties=[], skip_geometry=False, q=None, **kwargs):
@@ -155,6 +157,7 @@ class MongoProvider(BaseProvider):
 
         return feature_collection
 
+    @crs_transform
     def get(self, identifier, **kwargs):
         """
         query the provider by id
