@@ -2,7 +2,7 @@
 #
 # Authors: Tom Kralidis <tomkralidis@gmail.com>
 #
-# Copyright (c) 2020 Tom Kralidis
+# Copyright (c) 2022 Tom Kralidis
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -132,7 +132,7 @@ class RasterioProvider(BaseProvider):
 
         for i, dtype, nodataval in zip(self._data.indexes, self._data.dtypes,
                                        self._data.nodatavals):
-            LOGGER.debug('Determing rangetype for band {}'.format(i))
+            LOGGER.debug(f'Determing rangetype for band {i}')
 
             name, units = None, None
             if self._data.units[i-1] is None:
@@ -146,12 +146,11 @@ class RasterioProvider(BaseProvider):
                 'type': 'Quantity',
                 'name': name,
                 'encodingInfo': {
-                    'dataType': 'http://www.opengis.net/def/dataType/OGC/0/{}'.format(dtype)  # noqa
+                    'dataType': f'http://www.opengis.net/def/dataType/OGC/0/{dtype}'  # noqa
                 },
                 'nodata': nodataval,
                 'uom': {
-                    'id': 'http://www.opengis.net/def/uom/UCUM/{}'.format(
-                         units),
+                    'id': f'http://www.opengis.net/def/uom/UCUM/{units}',
                     'type': 'UnitReference',
                     'code': units
                 },
@@ -162,11 +161,11 @@ class RasterioProvider(BaseProvider):
 
         return rangetype
 
-    def query(self, range_subset=[], subsets={}, bbox=[], datetime_=None,
-              format_='json', **kwargs):
+    def query(self, properties=[], subsets={}, bbox=None, bbox_crs=4326,
+              datetime_=None, format_='json', **kwargs):
         """
         Extract data from collection collection
-        :param range_subset: list of bands
+        :param properties: list of bands
         :param subsets: dict of subset names with lists of ranges
         :param bbox: bounding box [minx,miny,maxx,maxy]
         :param datetime_: temporal (datestamp or extent)
@@ -175,14 +174,17 @@ class RasterioProvider(BaseProvider):
         :returns: coverage data as dict of CoverageJSON or native format
         """
 
-        bands = range_subset
-        LOGGER.debug('Bands: {}, subsets: {}'.format(bands, subsets))
+        bands = properties
+        LOGGER.debug(f'Bands: {bands}, subsets: {subsets}')
 
         args = {
             'indexes': None
         }
         shapes = []
 
+        if not bbox:
+            bbox = []
+
         if all([not bands, not subsets, not bbox, format_ != 'json']):
             LOGGER.debug('No parameters specified, returning native data')
             return read_data(self.data)
@@ -197,9 +199,9 @@ class RasterioProvider(BaseProvider):
         if len(bbox) > 0:
             minx, miny, maxx, maxy = bbox
 
-            crs_src = CRS.from_epsg(4326)
+            crs_src = CRS.from_epsg(bbox_crs)
 
-            if 'crs' in self.options:
+            if self.options and 'crs' in self.options:
                 crs_dest = CRS.from_string(self.options['crs'])
             else:
                 crs_dest = self._data.crs
@@ -224,10 +226,8 @@ class RasterioProvider(BaseProvider):
                 minx2, miny2 = t.transform(minx, miny)
                 maxx2, maxy2 = t.transform(maxx, maxy)
 
-                LOGGER.debug('Source coordinates: {}'.format(
-                    [minx, miny, maxx, maxy]))
-                LOGGER.debug('Destination coordinates: {}'.format(
-                    [minx2, miny2, maxx2, maxy2]))
+                LOGGER.debug(f'Source coordinates: {minx}, {miny}, {maxx}, {maxy}')  # noqa
+                LOGGER.debug(f'Destination: {minx2}, {miny2}, {maxx2}, {maxy2}')  # noqa
 
                 shapes = [{
                    'type': 'Polygon',
@@ -368,7 +368,7 @@ class RasterioProvider(BaseProvider):
         else:
             bands_select = metadata['bands']
 
-        LOGGER.debug('bands selected: {}'.format(bands_select))
+        LOGGER.debug(f'bands selected: {bands_select}')
         for bs in bands_select:
             pm = _get_parameter_metadata(
                 self._data.profile['driver'], self._data.tags(bs))
@@ -434,10 +434,7 @@ class RasterioProvider(BaseProvider):
 
         if self._data.crs is not None:
             if self._data.crs.is_projected:
-                properties['bbox_crs'] = '{}/{}'.format(
-                    'http://www.opengis.net/def/crs/OGC/1.3/',
-                    self._data.crs.to_epsg())
-
+                properties['bbox_crs'] = f'http://www.opengis.net/def/crs/OGC/1.3/{self._data.crs.to_epsg()}'  # noqa
                 properties['x_axis_label'] = 'x'
                 properties['y_axis_label'] = 'y'
                 properties['bbox_units'] = self._data.crs.linear_units
