@@ -38,6 +38,7 @@ import json
 from pygeoapi.plugin import InvalidPluginError
 from pygeoapi.provider.base import (BaseProvider, ProviderConnectionError,
                                     ProviderItemNotFoundError)
+from pygeoapi.util import crs_transform
 
 LOGGER = logging.getLogger(__name__)
 
@@ -148,13 +149,17 @@ class SQLiteGPKGProvider(BaseProvider):
         if row_data:
             rd = dict(row_data)  # sqlite3.Row is doesnt support pop
             feature = {
-                'type': 'Feature'
+                'type': 'Feature',
+                'geometry': None
             }
-            feature["geometry"] = json.loads(
-                rd.pop(f'AsGeoJSON({self.geom_col})')
-                )
-            if skip_geometry:
-                feature["geometry"] = None
+
+            try:
+                if not skip_geometry:
+                    feature['geometry'] = json.loads(
+                        rd.pop(f'AsGeoJSON({self.geom_col})')
+                    )
+            except TypeError:
+                LOGGER.warning('Missing geometry')
 
             feature['properties'] = rd
             feature['id'] = feature['properties'].pop(self.id_field)
@@ -271,6 +276,7 @@ class SQLiteGPKGProvider(BaseProvider):
 
         return cursor
 
+    @crs_transform
     def query(self, offset=0, limit=10, resulttype='results',
               bbox=[], datetime_=None, properties=[], sortby=[],
               select_properties=[], skip_geometry=False, q=None, **kwargs):
@@ -330,6 +336,7 @@ class SQLiteGPKGProvider(BaseProvider):
 
         return feature_collection
 
+    @crs_transform
     def get(self, identifier, **kwargs):
         """
         Query the provider for a specific
@@ -337,7 +344,7 @@ class SQLiteGPKGProvider(BaseProvider):
 
         :param identifier: feature id
 
-        :returns: GeoJSON FeaturesCollection
+        :returns: dict of single GeoJSON feature
         """
 
         LOGGER.debug('Get item from SQLite/GPKG')
